#include "stdafx.h"

#include "ParseTree.h"

#include "parser.h"
#include "lexer.h"

#include "msg_definitions.h"
#include "conversion.h"

CParseTree::CParseTree()
{
}


CParseTree::~CParseTree()
{
}

void CParseTree::SayHello(char* const msg) const
{
	std::cout << "Msg from CParseTree: " << msg << std::endl;
}

void CParseTree::PutErrorMsg(const wchar_t* msg)
{
	this->m_msgvctr.emplace_back(msg);
}

void CParseTree::PutErrorMsg(const char* msg)
{
	std::ostringstream os;
	os << "At row " << this->m_cell_index <<" - "<< msg;
	this->m_msgvctr.emplace_back(CharToWString(const_cast<char*>(os.str().c_str())));
}

double CParseTree::GetResult()
{
	return this->m_Result;
}
const wstrvctr_t& CParseTree::GetMessages() const
{
	return this->m_msgvctr;
}

void CParseTree::PutResult(double rlt)
{
	this->m_Result = rlt;
}

int CParseTree::CalculateCell(int index, const std::wstring& code)
{
	try
	{
		this->m_cell_index = index;

		yyscan_t scanner;
		yylex_init_extra(this, &scanner);

		size_t size = code.size();
		// allocate 1 more character (2 bytes) more
		// for L'\0'
		wchar_t *buffer = new wchar_t[size+1];

		// this size is count of character
		wcscpy_s(buffer, size+1, code.c_str());
		
		// the last character should be L'\0'
		//buffer[size+1] = L'\0';
		
		// this size should be count of bytes
		yy_scan_buffer((char*)buffer, (size+1)*2, scanner);

		int rlt = yyparse(scanner, this); 

		yylex_destroy(scanner);

		delete[] buffer;

		return rlt;
	}
	catch (std::exception&)
	{
		return -1;
	}
}

int CParseTree::RunParser(const std::string& filename)
{
	int rlt;

	try
	{
		yyscan_t scanner;

		FILE *in = nullptr;  errno_t er;

		if (filename != "")
			er = fopen_s(&in, filename.c_str(), "r");
		else
			in = stdin;

		yylex_init_extra(this, &scanner);
		yyset_in(in, scanner);

		rlt = yyparse(scanner, this);

		yylex_destroy(scanner);

		if (in != nullptr)
		{
			fclose(in);
			in = nullptr;
		}
	}
	catch (std::exception&)
	{

	}

	return rlt;
}


void CParseTree::copy_cstr(char** pTarget, const char* pSource)
{
	size_t size = strlen(pSource) + 1;
	*pTarget = new char[size];
	strcpy_s(*pTarget, size, pSource);
}

void CParseTree::copy_int(int& Target, const char* pSource)
{
	Target = atoi(pSource);
}
